@@ -5,7 +5,6 @@ from ..transformersx_base import *
 @configclass
 class FastTaskArguments:
     action: str = field("train", "the task action: train,eval,predict")
-    task_name: str = field('default', 'input the task name')
     data_dir: str = field("/app/dataset", "the data directory for train,eval,test,label")
     model_finetuned_dir: str = field("/app/models/finetuning",
                                      "The 'model_base_dir' generally is for the finetuned models."
@@ -22,4 +21,4 @@ class FastTaskArguments:
     label_col: str = field('label', '')
     batch_size_per_gpu: int = field(16, '')
     learning_rate: float = field(5e-5, "The initial learning rate for Adam.")
-    num_train_epochs: float = field(3.0, "Total number of training epochs to perform.")
+    num_train_epochs: int = field(3, "Total number of training epochs to perform.")
