@@ -1,5 +1,5 @@
 import torch
-from fast_bert import accuracy_thresh, roc_auc, fbeta
+from fast_bert import accuracy_thresh, roc_auc, fbeta, accuracy
 
 from .fast_task_args import FastTaskArguments
 from fast_bert.learner_abs import BertAbsDataBunch, BertAbsLearner
@@ -11,24 +11,25 @@ from ..transformersx_base import *
 
 
 class TransformersFastTask():
-    def __int__(self, args: FastTaskArguments):
+    def __init__(self, task_name, args: FastTaskArguments):
         self._args = args
+        self._task_name = task_name
 
 
 class TransformersFastAbsTask(TransformersFastTask):
-    def __int__(self, args: FastTaskArguments):
-        super().__int__(args)
+    def __init__(self, task_name, args: FastTaskArguments):
+        super().__init__(task_name, args)
 
 
 class TransformersFastClsTask(TransformersFastTask):
-    def __int__(self, args: FastTaskArguments):
-        super().__int__(args)
+    def __init__(self, task_name, args: FastTaskArguments):
+        super().__init__(task_name, args)
         self._model_path = join_path(self._args.model_pretrained_dir, self._args.model_name)
-        self._output_dir = join_path(self._args.model_finetuned_dir, self._args.model_name)
+        self._output_dir = join_path(self._args.model_finetuned_dir, task_name, self._args.model_name)
 
     def metrics(self):
         metrics = []
-        metrics.append({'name': 'accuracy_thresh', 'function': accuracy_thresh})
+        metrics.append({'name': 'accuracy', 'function': accuracy})
         metrics.append({'name': 'roc_auc', 'function': roc_auc})
         metrics.append({'name': 'fbeta', 'function': fbeta})
         return metrics
@@ -57,10 +58,10 @@ class TransformersFastClsTask(TransformersFastTask):
 
 
 class TransformersFastLmTask(TransformersFastTask):
-    def __int__(self, args: FastTaskArguments):
-        super().__int__(args)
+    def __init__(self, task_name, args: FastTaskArguments):
+        super().__init__(task_name, args)
 
 
 class TransformersFastQaTask(TransformersFastTask):
-    def __int__(self, args: FastTaskArguments):
-        super().__int__(args)
+    def __init__(self, task_name, args: FastTaskArguments):
+        super().__init__(task_name, args)
