@@ -8,3 +8,8 @@ class TestTransformersModel:
         model = task_model("bert-base-chinese", ModelTaskType.seq_cls, )
         assert issubclass(model.config, (BertConfig))
         assert issubclass(model.model_class, BertForSequenceClassification)
+
+    def test_models(self):
+        models = query_base_models(["bert", "albert"])
+        assert models["bert"] and models["albert"]
+        assert models["bert"]["cn"][0].model_path.index("bert") > -1
